package math

import (
	"context"
	"github.com/MontFerret/ferret/pkg/runtime/core"
	"github.com/MontFerret/ferret/pkg/runtime/values"
	"math"
)

/*
 * Returns the nearest integer, rounding half away from zero.
 * @param number (Int|Float) - Input number.
 * @returns (Int) - The nearest integer, rounding half away from zero.
 */
func Round(_ context.Context, args ...core.Value) (core.Value, error) {
	err := core.ValidateArgs(args, 1, 1)

	if err != nil {
		return values.None, err
	}

	err = core.ValidateType(args[0], core.IntType, core.FloatType)

	if err != nil {
		return values.None, err
	}

	return values.NewInt(int(math.Round(toFloat(args[0])))), nil
}
